<?php

/**
 * Plugin Name: Affiliate Stripe Payments Integration
 * Plugin URI: https://www.tipsandtricks-hq.com/wordpress-affiliate/stripe-payments-plugin-affiliate-platform-integration-1457
 * Description: Addon for integrating affiliate platform with the Stripe Payments WordPress Plugin.
 * Version: 1.7
 * Author: Tips and Tricks HQ
 * Author URI: https://www.tipsandtricks-hq.com/
 * Requires at least: 3.0
 */

//slug - aspi_

if (!defined('ABSPATH')){
    exit;
}

/************************************/
/* Stripe Subscription Addon related */
/************************************/
function process_asp_subcription_invoice_paid($sub_post_id, $payment_data) {
    wp_affiliate_log_debug("asp_subscription_invoice_paid hook triggered. Sub ID: " . $sub_post_id, true);
    
    //Test code
    $test_ap_id = get_post_meta( $sub_post_id, 'ap_id', true );
    wp_affiliate_log_debug("POST Meta AP ID: " . $test_ap_id, true);
    //End test code
    
    $ap_id = get_post_meta( $sub_post_id, 'ap_id', true );
    if (empty($ap_id)){
        //Try to get from cookie
        $ap_id = isset( $_COOKIE['ap_id'] ) ? $_COOKIE['ap_id'] : '';
    }
    $customer_ip_address = get_post_meta( $sub_post_id, 'customer_ip_address', true );
    wp_affiliate_log_debug("Customer IP address: " . $customer_ip_address . ", Aff ID: " . $ap_id, true);
    if(empty($ap_id) && !empty($customer_ip_address)){
        //Try to get using IP address
        wp_affiliate_log_debug("Stripe Payments integration - Checking affiliate ID using customer IP address.", true);
        $ap_id = wp_aff_get_referrer_id_from_ip_address($customer_ip_address);
    }    
    
    //$stripe_sub_id = get_post_meta( $sub_post_id, 'sub_id', true );
    //$sub = aspi_find_sub( $stripe_sub_id );

    if (!empty($ap_id)) {
        //Aff platform referral exist for this subscription.
        wp_affiliate_log_debug("Stripe Subscription - Affiliate ID: " . $ap_id, true);

        $amount_paid = isset( $payment_data['amount_paid'] ) ? $payment_data['amount_paid'] : 0;
        $sub_total_excl_tax = isset( $payment_data['subtotal_excluding_tax'] ) ? $payment_data['subtotal_excluding_tax'] : 0;
        
        wp_affiliate_log_debug("Amount paid: " . $amount_paid . ", Sub total excl tax: " . $sub_total_excl_tax, true);
        
//        if ( ! AcceptStripePayments::is_zero_cents( $curr ) ) {
//            $amount = round (intval( $sub_total_excl_tax ) / 100 );
//        }
        $amount = round (intval( $sub_total_excl_tax ) / 100 );
        
        $stripe_sub_id = get_post_meta( $sub_post_id, 'sub_id', true );
        $email = get_post_meta( $sub_post_id, 'customer_email', true );
        $plan_post_id = get_post_meta( $sub_post_id, 'plan_post_id', true );
        $reference = 'Plan ID: ' . $plan_post_id;
        
        wp_affiliate_log_debug('Stripe subscription - Amount: '.$amount.', Subscription ID: '.$stripe_sub_id.', Email: '.$email.', ap_id: '. $ap_id, true);

        wp_aff_award_commission_unique($ap_id, $amount, $stripe_sub_id, $reference, $email);
        wp_affiliate_log_debug("Stripe subscription payment - Commission awarded!", true);
    }
}
add_action( 'asp_subscription_invoice_paid', 'process_asp_subcription_invoice_paid', 10, 2);

function aspi_find_sub( $stripe_sub_id ) {
        $sub = get_posts(
                array(
                        'post_type'   => ASPSUB_main::$subs_slug,
                        'meta_key'    => 'sub_id',
                        'meta_value'  => $stripe_sub_id, /* sub ID of stripe (not Post ID) */
                        'post_status' => array( 'publish', 'pending' ),
                )
        );
        if ( $sub ) {
                return $sub[0];
        } else {
                return false;
        }
}
/***********************************/
/* End subscription addon related */
/***********************************/


/***********************************/
/* Standard Stripe Payments Charge */
/***********************************/
// Takes the $post_data and $charge as a parameter so you can pull information from the charge
// For charge response info see: https://stripe.com/docs/api#charge_object
function affiliate_stripe_payments_after_charge( $post_data, $charge ) {
    // Useful for adding additional functionality after the charge is complete

    if ( !function_exists('wp_affiliate_log_debug') ){
        //The main affiliate plugin is not active. Bail out.
        return;
    }

    wp_affiliate_log_debug('Stripe Payments plugin integration - asp_stripe_payment_completed hook triggered.', true);

    //Required Parameters
    $item_price = $post_data['item_price'];//Sale amount
    $item_quantity = isset($post_data['item_quantity']) ? $post_data['item_quantity'] : 1;//Item quantity
    $tax_amt = isset($post_data['tax']) ? $post_data['tax'] : 0;//Tax amount if any;
    $shipping_amt = isset($post_data['shipping']) ? $post_data['shipping'] : 0;//Shipping amount if any;

    $order_id = $post_data['txn_id'];//Transaction ID
    $email = $post_data['stripeEmail'];//Email address

    //Optional parameters
    $reference = $post_data['item_name'];//Use the item name as the reference
    $ip_address = isset($charge->client_ip) ? $charge->client_ip : '';

    //Check the referrer data
    $ap_id = $_COOKIE['ap_id'];
    if(empty($ap_id) && !empty($ip_address)){
        wp_affiliate_log_debug("Stripe Payments integration - Checking affiliate ID using customer IP address.", true);
        $ap_id = wp_aff_get_referrer_id_from_ip_address($ip_address);
    }
    if (empty($ap_id)) {
        wp_affiliate_log_debug("Stripe Payments integration - affiliate ID is not present. This customer was not sent by an affiliate.", true);
        return;
    }

    //Check if coupon code was used and if it mataches any affiliate ID
    if (isset($post_data['coupon']) && !empty($post_data['coupon'])){
        wp_affiliate_log_debug('Stripe Payments integration - Coupon Code was used in this transaction. Checking if it matches with any affiliate ID.', true);
        $aff_id = aspi_check_if_coupon_matches_aff_id($post_data);
        if( !empty($aff_id) ){
            //Override the affiliate ID
            $ap_id = $aff_id;
            wp_affiliate_log_debug('Overriding the affiliate commission to go to this affiliate: '. $ap_id, true);
        }
    }

    //Amount calculation (for commission purpose)
    wp_affiliate_log_debug('Stripe Payments integration - Item price: '.$item_price.', Item quantity: '.$item_quantity.', Tax: '.$tax_amt.', Shipping: '. $shipping_amt, true);
    $amount = ($item_price * $item_quantity) - $tax_amt - $shipping_amt;

    wp_affiliate_log_debug('Stripe Payments integration - Amount: '.$amount.', Order ID: '.$order_id.', Email: '.$email.', ap_id: '. $ap_id, true);

    wp_aff_award_commission_unique($ap_id, $amount, $order_id, $reference, $email);
    wp_affiliate_log_debug("Stripe Payments integration - Commission awarded!", true);
}
add_action( 'asp_stripe_payment_completed', 'affiliate_stripe_payments_after_charge', 10, 2 );

/*
 * Utility Functions
 */
function aspi_check_if_coupon_matches_aff_id($post_data){
    $aff_id = '';

    $coupon_code = isset($post_data['coupon']['code']) ? $post_data['coupon']['code'] : '';
    wp_affiliate_log_debug('Coupon code used is: ' . $coupon_code, true);

    //Lets see if an affiliate record exists with the affiliate ID same as the coupon code
    $resultset = wp_aff_get_affiliate_record_from_db($coupon_code);
    if( $resultset ){
        wp_affiliate_log_debug('Found a match. Affiliate ID: ' . $resultset->refid, true);
        $aff_id = $resultset->refid;
    }

    return $aff_id;
}